﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using System.IO;

namespace PDFToJPGExpert
{
    public partial class frmMain : PDFToJPGExpert.CustomForm
    {
        public static frmMain Instance = null;
        public DataTable dt = new DataTable("pdfs");
        public static System.IO.FileSystemWatcher fs = new System.IO.FileSystemWatcher();

        public string SelectedOutputFormatCode = "";
        public string SelectedOutputFormatExtension = "";
        public List<string> cmbOutputFormatCodes = new List<string>();
        public List<string> cmbOutputFormatExtension = new List<string>();
        
        public frmMain()
        {
            Instance = this;

            InitializeComponent();

            dt.Columns.Add("pagerange");
            dt.Columns.Add("totalpages", typeof(int));
            dt.Columns.Add("password");
            dt.Columns.Add("filedate");
            dt.Columns.Add("sizekb");
            dt.Columns.Add("fullfilepath");
            dt.Columns.Add("filename");
            dt.Columns.Add("decryptedfile");

            

        }

        private void frmMain_Load(object sender, EventArgs e)
        {
            SetupOnLoad();

            frmMain.fs.Created += fs_Created;
            frmMain.fs.Changed += fs_Changed;

            UpdateHelper.InitializeCheckVersion();
        }

        private void SetupOnLoad()
        {
            dgFiles.AutoGenerateColumns = false;
            dgFiles.DataSource = dt;

            this.Text = Module.ApplicationTitle;

            AdjustSizeLocation();
            
            SetupOutputFormats();

            cmbOutputFolder.Items.Add(TranslateHelper.Translate("Same as Document Folder"));
            cmbOutputFolder.Items.Add(TranslateHelper.Translate("<Other>"));
            cmbOutputFolder.SelectedIndex = 0;

            DownloadSuggestionsHelper ds = new DownloadSuggestionsHelper();
            ds.SetupDownloadMenuItems(tsiDownload);

            AddLanguageMenuItems();
        }

        #region Localization

        private void AddLanguageMenuItems()
        {
            for (int k = 0; k < frmLanguage.LangCodes.Count; k++)
            {
                ToolStripMenuItem ti = new ToolStripMenuItem();
                ti.Text = frmLanguage.LangDesc[k];
                ti.Tag = frmLanguage.LangCodes[k];
                ti.Image = frmLanguage.LangImg[k];
                ti.Click += new EventHandler(tiLang_Click);

                if (ti.Tag.ToString() == Properties.Settings.Default.Language)
                {
                    ti.Checked = true;
                }

                languageToolStripMenuItem.DropDownItems.Add(ti);
            }
        }

        void tiLang_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem ti = (ToolStripMenuItem)sender;
            string langcode = ti.Tag.ToString();
            ChangeLanguage(langcode);

            for (int k = 0; k < frmLanguage.LangCodes.Count; k++)
            {
                if (frmLanguage.LangCodes[k] == langcode)
                {
                    ToolStripMenuItem til = (ToolStripMenuItem)languageToolStripMenuItem.DropDownItems[k];
                    til.Checked = true;
                }
            }

            /*
            for (int k = 0; k < languageToolStripMenuItem.DropDownItems.Count; k++)
            {
                ToolStripMenuItem til = (ToolStripMenuItem)languageToolStripMenuItem.DropDownItems[k];
                if (til == ti)
                {
                    til.Checked = true;
                }
                else
                {
                    til.Checked = false;
                }
            }*/
        }

        private void ChangeLanguage(string language_code)
        {
            Properties.Settings.Default.Language = language_code;
            Program.SetLanguage();

            bool maximized = (this.WindowState == FormWindowState.Maximized);
            this.WindowState = FormWindowState.Normal;

            this.Controls.Clear();

            InitializeComponent();

            SetupOnLoad();

            if (maximized)
            {
                this.WindowState = FormWindowState.Maximized;
            }

            this.ResumeLayout(true);
        }

        #endregion

        private void SetupOutputFormats()
        {
            cmbOutputFormat.Items.Add("PNG");
            cmbOutputFormatCodes.Add("png16m");
            cmbOutputFormatExtension.Add("png");

            cmbOutputFormat.Items.Add("JPEG");
            cmbOutputFormatCodes.Add("jpeg");
            cmbOutputFormatExtension.Add("jpg");
            cmbOutputFormat.Items.Add("BMP");
            cmbOutputFormatExtension.Add("bmp");
            cmbOutputFormatCodes.Add("bmp16m");
            cmbOutputFormat.Items.Add("PCX");
            cmbOutputFormatExtension.Add("pcx");
            cmbOutputFormatCodes.Add("pcx24b");
            cmbOutputFormat.Items.Add("PSD");
            cmbOutputFormatExtension.Add("psd");
            cmbOutputFormatCodes.Add("psdrgb");
            cmbOutputFormat.Items .Add("TIFF G4 FAX Encoding");
            cmbOutputFormatCodes.Add("tiffg4");
            cmbOutputFormatExtension.Add("tif");
            cmbOutputFormat.Items.Add("TIFF LZW Compression");
            cmbOutputFormatCodes.Add("tifflzw");
            cmbOutputFormatExtension.Add("tif");
            cmbOutputFormat.Items.Add("TIFF 24 Bit uncompressed");
            cmbOutputFormatCodes.Add("tiff24nc");
            cmbOutputFormatExtension.Add("tif");
            cmbOutputFormat.Items.Add("JPEG Grayscale");
            cmbOutputFormatCodes.Add("jpeggray");
            cmbOutputFormatExtension.Add("jpg");
            cmbOutputFormat.Items.Add("PNG Grayscale");
            cmbOutputFormatCodes.Add("pnggray");
            cmbOutputFormatExtension.Add("png");
            cmbOutputFormat.Items.Add("PNG 256 Colors (8-bit)");
            cmbOutputFormatCodes.Add("png256");
            cmbOutputFormatExtension.Add("png");
            cmbOutputFormat.Items.Add("PNG 16 Colors (4-bit)");
            cmbOutputFormatCodes.Add("png16");
            cmbOutputFormatExtension.Add("png");
            cmbOutputFormat.Items.Add("PNG Monochrome");
            cmbOutputFormatCodes.Add("pngmono");
            cmbOutputFormatExtension.Add("png");

            /*
            cmbColors.Items.Add("16 Million Colors (24-bit)");
            cmbColors.Items.Add("256 Colors (8-bit)");
            cmbColors.Items.Add("16 Colors (4-bit)");
            cmbColors.Items.Add("Grayscale");
            cmbColors.Items.Add("Black-White (Monochrome)");
            */

            cmbOutputFormat.SelectedIndex = 0;
        }

        private void tsbConvert_Click(object sender, EventArgs e)
        {
            PDFToJPGHelper.Convert();
        }

        private void menuStrip1_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {

        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmAbout f = new frmAbout();
            f.ShowDialog();
        }

        private void tiHelpFeedback_Click(object sender, EventArgs e)
        {
            frmUninstallQuestionnaire f = new frmUninstallQuestionnaire(false);
            f.ShowDialog();
        }

        private void visit4dotsSoftwareWebsiteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start("http://www.4dots-software.com");
        }

        private void helpGuideToolStripMenuItem_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start(Module.HelpURL);

            ///System.Diagnostics.Process.Start(Application.StartupPath + "\\PDF To JPG Expert - User's Manual.chm");

            /*
            if (Module.SelectedLanguage == "en-US")
            {
                System.Diagnostics.Process.Start(Application.StartupPath + "\\PDF Password Remover Users Manual.chm");
            }
            else
            {
                System.Diagnostics.Process.Start(Application.StartupPath + "\\" + Module.SelectedLanguage + "\\PDF Password Remover Users Manual.chm");
            }
            */
        }

        private void tsbAddFile_Click(object sender, EventArgs e)
        {
            openFileDialog1.Filter = Module.DialogFilesFilter;
            openFileDialog1.Multiselect = true;

            openFileDialog1.FileName = "";
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    this.Cursor = Cursors.WaitCursor;

                    for (int k = 0; k < openFileDialog1.FileNames.Length; k++)
                    {
                        AddFile(openFileDialog1.FileNames[k]);
                    }
                }
                finally
                {
                    this.Cursor = null;
                }
            }
        }

        private void AddFile(string file)
        {
          GetPDFTotalPagesResult res= GeneratePDFImagesHelper.GetPDFTotalPages(file);

          if (res.ErrStr != string.Empty)
          {
              frmError f = new frmError(TranslateHelper.Translate("Error"), res.ErrStr);
              f.ShowDialog();
              return;
          }
          else
          {
              DataRow dr = dt.NewRow();
              dr["totalpages"] = res.TotalPages;
              dr["decryptedfile"] = res.DecryptedFile;
              dr["filename"] = System.IO.Path.GetFileName(file);
              dr["fullfilepath"] = file;

              FileInfo fi = new FileInfo(file);
              
              long sizekb=fi.Length / 1024;
              dr["sizekb"] = sizekb.ToString() + " KB";
              dr["filedate"] = fi.LastWriteTime.ToString();
              dr["password"] = res.Password;
              dr["pagerange"] = TranslateHelper.Translate("All Pages");

              dt.Rows.Add(dr);

          }
        }

        private void tsbAddFolder_Click(object sender, EventArgs e)
        {
            folderBrowserDialog1.SelectedPath = "";
            if (folderBrowserDialog1.ShowDialog() == DialogResult.OK)
            {
                AddFolder(folderBrowserDialog1.SelectedPath);
            }
        }

        public void AddFolder(string folder_path)
        {
            string[] filez = null;

            /*
            if (Module.IsCommandLine)
            {
                if (Module.CmdAddSubdirectories)
                {
                    filez = System.IO.Directory.GetFiles(folder_path, "*.pdf", SearchOption.AllDirectories);
                }
                else
                {
                    filez = System.IO.Directory.GetFiles(folder_path, "*.pdf", SearchOption.TopDirectoryOnly);
                }
            }
            else
            {
            */
                if (System.IO.Directory.GetDirectories(folder_path).Length > 0)
                {
                    DialogResult dres = Module.ShowQuestionDialog("Would you like to add also Subdirectories ?", "Add Subdirectories ?");

                    if (dres == DialogResult.Yes)
                    {
                        filez = System.IO.Directory.GetFiles(folder_path, "*.pdf", SearchOption.AllDirectories);
                    }
                    else
                    {
                        filez = System.IO.Directory.GetFiles(folder_path, "*.pdf", SearchOption.TopDirectoryOnly);
                    }
                }
                else
                {
                    filez = System.IO.Directory.GetFiles(folder_path, "*.pdf", SearchOption.TopDirectoryOnly);
                }
            //}

            try
            {
                this.Cursor = Cursors.WaitCursor;

                for (int k = 0; k < filez.Length; k++)
                {
                    if (filez[k].ToLower().EndsWith(".pdf"))
                    {
                        AddFile(filez[k]);
                    }

                }
            }
            finally
            {
                this.Cursor = null;
            }

        }

        private void tsbRemove_Click(object sender, EventArgs e)
        {
            DataGridViewSelectedCellCollection cells = dgFiles.SelectedCells;
            List<DataGridViewRow> rows = new List<DataGridViewRow>();

            for (int k = 0; k < cells.Count; k++)
            {
                if (rows.IndexOf(dgFiles.Rows[cells[k].RowIndex]) < 0)
                {
                    rows.Add(dgFiles.Rows[cells[k].RowIndex]);
                }
            }

            for (int k = 0; k < rows.Count; k++)
            {
                dgFiles.Rows.Remove(rows[k]);
            }
        }

        private void tsbClear_Click(object sender, EventArgs e)
        {
            dt.Rows.Clear();                        
        }

        private void bwProgress_DoWork(object sender, DoWorkEventArgs e)
        {                                               
            LastFilepathFired = "";

            PDFConvertArgs args = (PDFConvertArgs)e.Argument;
            try
            {
                args.con.Convert(args.InputFile, args.OutFile);
            }
            catch (Exception ex)
            {
                PDFToJPGHelper.errstr += TranslateHelper.Translate("Error while converting file") + " : " + args.InputFile + "\r\n" + ex.Message + "\r\n";
            }
            /*
            while (!bwProgress.CancellationPending)
            {
                Application.DoEvents();
                
            }
            */
                                                                        
        }

        private string LastFilepathFired = "";

        void fs_Changed(object sender, FileSystemEventArgs e)
        {
            //avoid duplicate fired events....
            if (LastFilepathFired != e.FullPath)
            {
                if (e.FullPath.StartsWith(PDFToJPGHelper.CurrentProcessingFilepathWithoutExtension)
                    && System.IO.Path.GetExtension(e.FullPath) == PDFToJPGHelper.CurrentProcessingFileExtension)
                {
                    //frmMain.Instance.bwProgress.ReportProgress(1);
                    frmProgress.Instance.UpdateProgress();
                    //MessageBox.Show("CHANGED!");
                }

                LastFilepathFired = e.FullPath;
            }
        }

        void fs_Created(object sender, FileSystemEventArgs e)
        {
            //avoid duplicate fired events....
            if (LastFilepathFired != e.FullPath)
            {
                if (e.FullPath.StartsWith(PDFToJPGHelper.CurrentProcessingFilepathWithoutExtension)
                    && System.IO.Path.GetExtension(e.FullPath) == PDFToJPGHelper.CurrentProcessingFileExtension)
                {
                    frmProgress.Instance.UpdateProgress();
                    //frmMain.Instance.bwProgress.ReportProgress(1);
                    //MessageBox.Show("CREATED!");
                }
                LastFilepathFired = e.FullPath;
            }
        }

        private void tsbEdit_Click(object sender, EventArgs e)
        {
            dgFiles_CellContentDoubleClick(null, null);
        }

        private void dgFiles_CellContentDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            if (dgFiles.SelectedCells == null || dgFiles.SelectedCells.Count == 0)
            {
                return;
            }

            DataRowView drv = (DataRowView)dgFiles.SelectedCells[0].OwningRow.DataBoundItem;
            DataRow dr = drv.Row;

            frmEditPDF f = new frmEditPDF(dr["filename"].ToString(), dr["password"].ToString(), dr["pagerange"].ToString(),dr["totalpages"].ToString());
            if (f.ShowDialog() == DialogResult.OK)
            {
                dr["password"] = f.txtPassword.Text;
                dr["pagerange"] = f.cmbPageRange.Text;
                //dr.AcceptChanges();
            }

        }

        private void btnOutputFolderSelect_Click(object sender, EventArgs e)
        {
            if (cmbOutputFolder.Text.Trim()!=string.Empty && System.IO.Directory.Exists(cmbOutputFolder.Text))
            {
                folderBrowserDialog1.SelectedPath=cmbOutputFolder.Text;
            }

            if (folderBrowserDialog1.ShowDialog() == DialogResult.OK)
            {
                bool found = false;

                for (int k = 0; k < cmbOutputFolder.Items.Count; k++)
                {
                    if (cmbOutputFolder.Items[k].ToString() == folderBrowserDialog1.SelectedPath)
                    {
                        found = true;
                        cmbOutputFolder.SelectedIndex = k;
                        return;
                    }
                }

                if (!found)
                {
                    cmbOutputFolder.Items.Add(folderBrowserDialog1.SelectedPath);
                    cmbOutputFolder.SelectedIndex = cmbOutputFolder.Items.Count - 1;
                }                
            }
        }

        private void btnOutputFolderOpen_Click(object sender, EventArgs e)
        {
            string outfolder = "";

            if (cmbOutputFolder.Text == TranslateHelper.Translate("Same as Document Folder"))
            {
                if (dgFiles.SelectedRows.Count == 0)
                {
                    Module.ShowMessage("Please select a PDF Document first !");
                    return;
                }

                string filepath = dgFiles.SelectedRows[0].Cells["colFullFilePath"].Value.ToString();
                outfolder = System.IO.Path.GetDirectoryName(filepath);

            }
            else if (cmbOutputFolder.Text.Trim() == string.Empty || !System.IO.Directory.Exists(cmbOutputFolder.Text))
            {
                Module.ShowMessage("Please specify a valid Output Folder !");
                return;
            }
            else
            {
                outfolder = cmbOutputFolder.Text;
            }
            
            //string args = string.Format("/e, /select, \"{0}\"", txtOutputFolder.Text);
            string args = string.Format("\"{0}\"", outfolder);
            ProcessStartInfo info = new ProcessStartInfo();
            info.FileName = "explorer";
            info.UseShellExecute = true;
            info.Arguments = args;
            Process.Start(info);
        }

        private void rdDimensionsOther_CheckedChanged(object sender, EventArgs e)
        {
            txtDimensionsH.Visible = rdDimensionsOther.Checked;
            txtDimensionsW.Visible = rdDimensionsOther.Checked;
            lblDimensions.Visible = rdDimensionsOther.Checked;
                        
        }

        private bool IsDragging = false;

        private void dgFiles_DragDrop(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop, false))
            {
                string[] filez = (string[])e.Data.GetData(DataFormats.FileDrop);

                for (int k = 0; k < filez.Length; k++)
                {
                    try
                    {
                        this.Cursor = Cursors.WaitCursor;
                        AddFile(filez[k]);
                    }
                    finally
                    {
                        this.Cursor = null;
                    }
                }
            }

        }

        private void dgFiles_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop, false))
            {
                IsDragging = true;
                e.Effect = DragDropEffects.All;

            }
            else
            {
                e.Effect = DragDropEffects.None;
            }

        }

        private void dgFiles_DragOver(object sender, DragEventArgs e)
        {
            IsDragging = true;
            if ((e.AllowedEffect & DragDropEffects.Copy) == DragDropEffects.Copy)
            {
                e.Effect = DragDropEffects.Copy;
            }
        }

        private void dgFiles_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            dgFiles_CellContentDoubleClick(null, null);
        }

        private void groupBox1_Enter(object sender, EventArgs e)
        {

        }

        private void btnOutputFormatMore_Click(object sender, EventArgs e)
        {
            frmOutputFormatSelect f = new frmOutputFormatSelect();
            if (f.ShowDialog() == DialogResult.OK)
            {
                SelectedOutputFormatCode = f.SelectedOutputFormatCode;
                SelectedOutputFormatExtension = f.SelectedOutputFormatExtension;
                cmbOutputFormat.Text = f.SelectedOutputFormat;
            }
        }

        private void cmbOutputFormat_SelectedIndexChanged(object sender, EventArgs e)
        {
            SelectedOutputFormatCode = cmbOutputFormatCodes[cmbOutputFormat.SelectedIndex];
            SelectedOutputFormatExtension = cmbOutputFormatExtension[cmbOutputFormat.SelectedIndex];

        }

        private void tsbConvert_Click_1(object sender, EventArgs e)
        {
            try
            {
                toolStrip1.Enabled = false;

                string errstr = PDFToJPGHelper.Convert();

                if (PDFToJPGHelper.CANCELLED)
                {
                    Module.ShowMessage("Operation Cancelled !");
                }
                else
                {
                    if (errstr == String.Empty)
                    {
                        Module.ShowMessage("Operation completed successfully !");
                    }
                    else
                    {
                        frmError f = new frmError(TranslateHelper.Translate("Operation completed with Errors !"), errstr);
                        f.ShowDialog();
                    }
                }
            }
            finally
            {
                toolStrip1.Enabled = true;
            }
        }

        private void rd800600_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void rd1024768_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void rd12801024_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void checkVersionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            UpdateHelper.CheckVersion(false);
        }

        private void AdjustSizeLocation()
        {
            if (Properties.Settings.Default.Maximized)
            {
                this.WindowState = FormWindowState.Maximized;
            }
            else
            {

                if (Properties.Settings.Default.Width == -1)
                {
                    this.CenterToScreen();
                    return;
                }
                else
                {
                    this.Width = Properties.Settings.Default.Width;
                }
                if (Properties.Settings.Default.Height != 0)
                {
                    this.Height = Properties.Settings.Default.Height;
                }

                if (Properties.Settings.Default.Left != -1)
                {
                    this.Left = Properties.Settings.Default.Left;
                }

                if (Properties.Settings.Default.Top != -1)
                {
                    this.Top = Properties.Settings.Default.Top;
                }
            }

        }

        private void SaveSizeLocation()
        {
            Properties.Settings.Default.Maximized = (this.WindowState == FormWindowState.Maximized);
            Properties.Settings.Default.Left = this.Left;
            Properties.Settings.Default.Top = this.Top;
            Properties.Settings.Default.Width = this.Width;
            Properties.Settings.Default.Height = this.Height;
            Properties.Settings.Default.Save();

        }

        private void frmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            SaveSizeLocation();
        }

        private void bwProgress_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            MessageBox.Show("PROGRESS CHANGED");

            if ((frmProgress.Instance.progressBar1.Value + 1) < frmProgress.Instance.progressBar1.Maximum)
            {
                frmProgress.Instance.progressBar1.Value += 1;
            }
        }

        
    }
}
